/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "EndPoint_UDP.h"

#include <stdio.h>

void EndPoint_UDP::SetAddress(const sockaddr_in*in_Address)
{
	memset (&m_Address, 0, sizeof(m_Address));
	memcpy (&m_Address.sin_addr, &in_Address->sin_addr, sizeof(in_Address->sin_addr));
	m_Address.sin_family = in_Address->sin_family;
	m_Address.sin_port = in_Address->sin_port;
	m_bValid=true;
}

void EndPoint_UDP::SetAddress(const char*in_strAddress, const int in_DefaultPort)
{
	char*strPort;
	char strAddress[256];
	int Port;

	memset (&m_Address, 0, sizeof(m_Address));
	m_bValid=false;
	strcpy(strAddress, in_strAddress);
	strPort=strstr(strAddress, ":");
	if(strPort != NULL)
	{
		*strPort='\0';
		Port=atoi(strPort+1);
	}
	else
		Port=in_DefaultPort;

	if(Port<=0 || Port > 65536)
		return;

	hostent *HostEntry = gethostbyname(strAddress);
	if (HostEntry == NULL)
	{
		printf("Could not find host : %s.\n", strAddress);
		return;
	}
	memcpy (&m_Address.sin_addr, HostEntry->h_addr, HostEntry->h_length);

	m_Address.sin_family = AF_INET;
	m_Address.sin_port = htons (Port);

	m_bValid=true;
}

void EndPoint_UDP::SetLocalPort(const char*in_strPort)
{
	SetLocalPort(atoi(in_strPort));
}

void EndPoint_UDP::SetLocalPort(const int in_Port)
{
	memset (&m_Address, 0, sizeof(m_Address));
	m_bValid=false;
	if(in_Port == -1 || (in_Port>0 && in_Port<65536))
	{
		m_Address.sin_family = AF_INET;
		if(in_Port != -1)
			SetPort(in_Port);
		m_Address.sin_addr.s_addr = htonl(INADDR_ANY);
		m_bValid=true;
	}
}

void EndPoint_UDP::SetPort(const int in_Port)
{
	m_Address.sin_port = htons(in_Port);
}

int EndPoint_UDP::GetPort() const
{
	return IsValid() ? ntohs(m_Address.sin_port) : 0;
}

int EndPoint_UDP::BindSocketLocal(int in_Socket) const
{
	if(!IsValid()||in_Socket<=0)
		return 1;
	int ret = bind(in_Socket, (sockaddr *)&m_Address, sizeof(m_Address));
	if( ret == 0 )
	{
		int AddrLen = sizeof(m_Address);
		getsockname(in_Socket, (sockaddr *)&m_Address, &AddrLen);
	} 
	return ret;
}

bool EndPoint_UDP::HasSockAddress(const sockaddr*in_RemoteAddress, const int in_AddrLen) const
{
	return in_AddrLen == sizeof(m_Address) && m_Address.sin_port != 0
	                                       && m_Address.sin_port == ((sockaddr_in*)in_RemoteAddress)->sin_port
	                                       && m_Address.sin_family == ((sockaddr_in*)in_RemoteAddress)->sin_family
	                                       && m_Address.sin_addr.s_addr == ((sockaddr_in*)in_RemoteAddress)->sin_addr.s_addr;
}

bool EndPoint_UDP::HasSameAddress(const EndPoint*in_EndPoint) const
{
	if( !in_EndPoint->GetType() == EPT_UDP )
		return false;
	return IsValid() && in_EndPoint->IsValid() && HasSockAddress( (const sockaddr*)&dynamic_cast<const EndPoint_UDP*>(in_EndPoint)->m_Address, sizeof(sockaddr_in) );
}

int EndPoint_UDP::SendDataWith(int in_Socket, const char*in_Buf, const int in_NumBuf)
{
	if(!IsValid()||in_Socket<=0)
		return 1;

	sockaddr_in LocalAddr;
	int AddrLen = sizeof(sockaddr_in);
	getsockname(in_Socket, (sockaddr*)&LocalAddr, &AddrLen);
	char SrcAddr[128];
	strcpy(SrcAddr, inet_ntoa(LocalAddr.sin_addr));
	return sendto(in_Socket, in_Buf, in_NumBuf, 0, (sockaddr *)&m_Address, sizeof(m_Address));
}

void EndPoint_UDP::GetText(char*out_strText, const int in_BufLen) const  
{
	if(!IsValid())
		strncpy(out_strText, "No UDP Address", in_BufLen);
	else
		snprintf(out_strText, in_BufLen, "%s:%d",inet_ntoa (m_Address.sin_addr), ntohs (m_Address.sin_port));
}

EndPoint_Type EndPoint_UDP::GetType() const
{
	return EPT_UDP;
}

EndPoint*EndPoint_UDP::Clone() const
{
	EndPoint_UDP*NewEndPoint = new EndPoint_UDP;
	NewEndPoint->SetAddress(&m_Address);
	return NewEndPoint;
}
