/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "ForwardedLink.h"
#include "ProxyBase.h"

#include <mem.h>
#include <stdio.h>

void ForwardedLink::ForwardData(const char*in_FwdBuf, const int in_NumFwdBuf)
{
	RemoteLink*NextLink = GetParentProxy()->GetNextLinkInRoute( m_ForwardLANID, m_ForwardLinkID );
	if( NextLink != NULL )
	{
		//printf("%s: Forwarding a %dB message to %d/%d (%s).\n", GetName(), in_NumFwdBuf, m_ForwardLANID, m_ForwardLinkID, NextLink->GetName());
		NextLink->SendForwardedData( QP_LAN_ID_SELF, m_LinkID, m_ForwardLANID, m_ForwardLinkID, -1, in_FwdBuf, in_NumFwdBuf );
	}
	else
		printf("%s: Couldn't find a route to forward a %dB message to %d/%d. Dropping.\n", GetName(), in_NumFwdBuf, m_ForwardLANID, m_ForwardLinkID);
}

ForwardedLink::ForwardedLink(const char*in_Name, LinkSetBase*in_ParentLinkSet, unsigned int in_LinkID, const EndPoint*in_LocalEndPoint, EndPoint*in_RemoteEndPoint)
: RemoteLink(in_Name, in_ParentLinkSet, in_LinkID, in_LocalEndPoint, in_RemoteEndPoint),
  m_NumFwdBuffer(0), m_MaxFwdBuffer(65536), m_FwdBuffer(NULL),
  m_ForwardLANID(QP_LAN_ID_INVALID), m_ForwardLinkID(QP_LINK_ID_INVALID)
{
	m_FwdBuffer = new char[m_MaxFwdBuffer];
	GetParentProxy()->AddRoutingEntry(QP_LAN_ID_SELF, m_LinkID, this);
}

ForwardedLink::~ForwardedLink()
{
	if(m_FwdBuffer)
		delete[]m_FwdBuffer;
}

void ForwardedLink::SetForwardingDestination(unsigned int in_LANID, unsigned int in_LinkID)
{
	m_ForwardLANID  = in_LANID;
	m_ForwardLinkID = in_LinkID;
	//printf("%s: Forwarding destination set to %d/%d.\n", GetName(), m_ForwardLANID, m_ForwardLinkID);
}

void ForwardedLink::ClearForwardData()
{
	m_NumFwdBuffer = 0;
}

void ForwardedLink::GetText(char*out_strText, const int in_BufLen) const  
{
	RemoteLink::GetText(out_strText, in_BufLen);
	if(IsValid())
		snprintf(out_strText, in_BufLen, "%s Fwd=%d/%d.", out_strText, m_ForwardLANID, m_ForwardLinkID);
}

void ForwardedLink::GetForwardData(char*&out_FwdData, int&out_NumFwdData)
{
	out_FwdData = m_FwdBuffer;
	out_NumFwdData = m_NumFwdBuffer;
}

