/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _LISTENINGLINKSET_H_
#define _LISTENINGLINKSET_H_

#include "LinkSet.h"

template<class RemoteLinkType>
class ListeningLinkSet : public LinkSet<RemoteLinkType>
{
	protected:
		unsigned int m_ForwardLANID;
		unsigned int m_ForwardLinkID;
		virtual void HandleDataFromUnknownAddress(const EndPoint*in_RemoteEndPoint, char*in_Buf, const int in_NumBuf);
	public:
		ListeningLinkSet(const char*in_Name, ProxyBase*in_ParentProxy, const EndPoint*in_ListenEndPoint, const int in_MaxLinks = 32, unsigned int in_ForwardLANID = QP_LAN_ID_INVALID, unsigned int in_ForwardLinkID = QP_LINK_ID_INVALID);
		virtual const char*GetTypeName() const;
};

template<class RemoteLinkType>
void ListeningLinkSet<RemoteLinkType>::HandleDataFromUnknownAddress(const EndPoint*in_RemoteEndPoint, char*in_Buf, const int in_NumBuf)
{
	this->AddRemoteLinkTo(in_RemoteEndPoint);
	if( m_ForwardLANID != QP_LAN_ID_INVALID && m_ForwardLinkID != QP_LINK_ID_INVALID )
		this->SetUpForwarding( in_RemoteEndPoint, m_ForwardLANID, m_ForwardLinkID ); 
	this->HandleDataFromAddress(in_RemoteEndPoint, in_Buf, in_NumBuf, true);
}

template<class RemoteLinkType>
ListeningLinkSet<RemoteLinkType>::ListeningLinkSet(const char*in_Name, ProxyBase*in_ParentProxy, const EndPoint*in_ListenEndPoint, const int in_MaxLinks, unsigned int in_ForwardLANID, unsigned int in_ForwardLinkID)
: LinkSet<RemoteLinkType>(in_Name, in_ParentProxy, in_ListenEndPoint, in_MaxLinks),
  m_ForwardLANID(in_ForwardLANID), m_ForwardLinkID(in_ForwardLinkID)
{
}

template<class RemoteLinkType>
const char*ListeningLinkSet<RemoteLinkType>::GetTypeName() const
{
	return "ListeningLinkSet";
}

#endif // _LISTENINGLINKSET_H_
