/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "MultiLinkSets.h"
#include "LinkSetBase.h"

#include <string.h>

MultiLinkSets::MultiLinkSets(const char*in_Name, ProxyBase*in_ParentProxy, const int in_MaxLinkSets)
: m_LinkSets(NULL), m_NumLinkSets(0), m_MaxLinkSets(in_MaxLinkSets), m_LinkSetNameCounter(0), m_ParentProxy(in_ParentProxy)
{
	strncpy(m_Name, in_Name, QP_MAX_NAME);
	m_LinkSets = new LinkSetBase*[m_MaxLinkSets];
}

const char*MultiLinkSets::GetName() const
{
	return m_Name;
}

void MultiLinkSets::Work()
{
	for(int i=0; i<m_NumLinkSets; i++)
		if(m_LinkSets[i] != NULL)
			m_LinkSets[i]->Work();
}

void MultiLinkSets::GetText(char*out_strText, const int in_BufLen) const
{
	char LinkSetText[2048];
	
	snprintf(out_strText, in_BufLen, "%s [MultiLinkSets]: %d/%d LinkSets.\n", GetName(), m_NumLinkSets, m_MaxLinkSets);
	for(int i=0; i<m_NumLinkSets; i++)
	{
		if(m_LinkSets[i] != NULL)
			m_LinkSets[i]->GetText(LinkSetText, 2048);
		snprintf(out_strText, in_BufLen, "%sLinkSet %d: %s\n", out_strText, i, m_LinkSets[i]?LinkSetText:"NONE");
	}
}

MultiLinkSets::~MultiLinkSets()
{
	if(m_LinkSets != NULL)
	{
		for(int i=0; i<m_NumLinkSets; i++)
			if(m_LinkSets[i] != NULL)
				delete m_LinkSets[i];
		delete[] m_LinkSets;
	}
}

