/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _MULTILINKSETS_H_
#define _MULTILINKSETS_H_

#include "QP_Network.h"
#include "LinkSetBase.h"
#include <stdio.h>

class LinkSetBase;
class EndPoint;
class ProxyBase;
class RemoteLink;

class MultiLinkSets
{
	private:
		char         m_Name[QP_MAX_NAME];
		LinkSetBase**m_LinkSets;
		int          m_NumLinkSets;
		const int    m_MaxLinkSets;
		int          m_LinkSetNameCounter;
		ProxyBase   *m_ParentProxy;
	public:
		MultiLinkSets(const char*in_Name, ProxyBase*in_ParentProxy, const int in_MaxLinkSets);
		template<class LinkSetType>
		RemoteLink*AddLink(const EndPoint*in_LocalEndPoint, const EndPoint*in_RemoteEndPoint);
		virtual ~MultiLinkSets();
		void Work();
		const char*GetName() const;
		void GetText(char*out_strText, const int in_BufLen) const;
};

template<class LinkSetType>
RemoteLink*MultiLinkSets::AddLink(const EndPoint*in_LocalEndPoint, const EndPoint*in_RemoteEndPoint)
{
	for(int i=0; i<m_NumLinkSets; i++)
		if( m_LinkSets[i] != NULL && m_LinkSets[i]->HasLocalEndPoint(in_LocalEndPoint) )
			return m_LinkSets[i]->GetRemoteLinkByNum(m_LinkSets[i]->AddRemoteLinkTo( in_RemoteEndPoint ));

	if( m_NumLinkSets < m_MaxLinkSets )
	{
		char LinkSetName[QP_MAX_NAME];
		snprintf(LinkSetName, QP_MAX_NAME, "%s.LS%02d", GetName(), m_LinkSetNameCounter++);
		m_LinkSets[m_NumLinkSets] = new LinkSetType( LinkSetName, m_ParentProxy, in_LocalEndPoint );
		m_NumLinkSets++;
		return m_LinkSets[m_NumLinkSets-1]->GetRemoteLinkByNum(m_LinkSets[m_NumLinkSets-1]->AddRemoteLinkTo( in_RemoteEndPoint ));
	}
	return NULL;
}

#endif // _MULTILINKSETS_H_
