/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "QuakeLink.h"
#include "EndPoint_UDP.h"
#include "LinkSetBase.h"
#include "ProxyBase.h"

#include <stdio.h>

void QuakeLink::HexDisplay(const char*in_Buf, int in_Num)
{
	const char*pBuf = in_Buf, *pBuf2 = in_Buf;
	while( pBuf < in_Buf+in_Num ) 
	{
		const char*pEndLine = pBuf + 16;
		printf("\t[");
		while ( pBuf < pEndLine )
		{
			if( pBuf < in_Buf+in_Num )
				printf("%02X", (int)*((unsigned char*)pBuf));
			else
				printf("  ");
			if( (*((int*)(&pBuf))-*((int*)(&in_Buf)) +1)%2 == 0 && pBuf != pEndLine)
				printf(" ");
			if( (*((int*)(&pBuf))-*((int*)(&in_Buf)) +1)%4 == 0 && pBuf != pEndLine)
				printf(" ");
			pBuf++;
		}
		printf("]  [");
		while ( pBuf2 < pEndLine )
		{
			if( pBuf2 < in_Buf+in_Num )
				printf("%c", *pBuf2);
			else
				printf(" ");
			if( (*((int*)(&pBuf2))-*((int*)(&in_Buf)) +1)%8 == 0 && pBuf2 != pEndLine)
				printf(" ");
			pBuf2++;
		}
		printf("]\n");
	}
}

int QuakeLink::GetPacketLength( const char*in_Data, int in_NumData ) 
{
	if ( in_NumData < 4 )
		return 0;
	int MsgHeader = ntohl(*((int*)in_Data)); 
	int MsgSize   = MsgHeader & 0x0000ffff;
	return MsgSize;
}

void QuakeLink::ProcessReceivedDataInternal(char*in_Buf, int&in_NumBuf, bool in_bPrintOnly)
{
	const char*DebugVerb=in_bPrintOnly?"Sending":"Got";
	while( in_NumBuf > 0 ) 
	{
		int MsgHeader = ntohl(*((int*)in_Buf)); 
		int MsgSize   = MsgHeader & 0x0000ffff;
		if( MsgSize > in_NumBuf )
		{
			printf("%s: %s partial (%d/%dB) message (Queuing):\n", GetName(), DebugVerb, in_NumBuf, MsgSize);
			//HexDisplay( in_Buf, in_NumBuf );
			//in_NumBuf = 0;
			return;
		}
		else
		{
			if( MsgHeader & (0x80000000) ) // CTL
			{
				unsigned char MsgCode = in_Buf[4];
				switch(MsgCode)
				{
					case 0x01: // Connect
						printf("%s: %s connect message - Game name is \"%s\".%s\n", GetName(), DebugVerb, in_Buf+5, in_bPrintOnly?"":" (Forwarding)");
						if(!in_bPrintOnly)
							ForwardData( in_Buf, in_NumBuf );
						break;
					case 0x81: // Accept
					{
						const int QUAKE_MSG_ACCEPT_PORT_OFFSET=5;
						int AccPort = (*((int*)(in_Buf+QUAKE_MSG_ACCEPT_PORT_OFFSET)));
						printf("%s: %s accept message, assigned port is %d.%s\n", GetName(), DebugVerb, AccPort, in_bPrintOnly?"":" (Forwarding)");
						if(!in_bPrintOnly)
						{
							EndPoint_UDP*RemoteEndPoint = (EndPoint_UDP*)m_RemoteEndPoint->Clone();
							RemoteEndPoint->SetPort(AccPort);
							RemoteLink*NewQuakeLink = m_ParentLinkSet->GetRemoteLinkByNum(m_ParentLinkSet->AddRemoteLinkTo(RemoteEndPoint));
							delete RemoteEndPoint;
	
							RemoteLink*NextLink = GetParentProxy()->GetNextLinkInRoute( m_ForwardLANID, m_ForwardLinkID );
							if( NextLink != NULL )
								NextLink->SendForwardedData( QP_LAN_ID_SELF, NewQuakeLink->GetLinkID(), m_ForwardLANID, m_ForwardLinkID, QUAKE_MSG_ACCEPT_PORT_OFFSET, in_Buf, in_NumBuf );
						}
					}
						break;
					default:
						//printf("%s: %s UNKNOWN %dB control message with code %d.%s\n", GetName(), DebugVerb, MsgSize, (unsigned int)MsgCode, in_bPrintOnly?"":" (Forwarding)");
						if(!in_bPrintOnly)
							ForwardData( in_Buf, in_NumBuf );
						break;
				}
			}
			else
			{
				//printf("%s: %s a %dB non-control message%s:\n", GetName(), DebugVerb, MsgSize, in_bPrintOnly?"":" (Forwarding)");
				//HexDisplay( in_Buf, MsgSize );
				if(!in_bPrintOnly)
					ForwardData( in_Buf, MsgSize );
			}
			if( in_NumBuf > MsgSize )
				memmove( in_Buf, in_Buf + MsgSize, in_NumBuf - MsgSize );
			in_NumBuf -= MsgSize;
		}
	}
}

void QuakeLink::ProcessReceivedData()
{
	ProcessReceivedDataInternal(m_RecvBuffer, m_NumRecvBuffer, false);
}

QuakeLink::QuakeLink(const char*in_Name, LinkSetBase*in_ParentLinkSet, unsigned int in_LinkID, const EndPoint*in_LocalEndPoint,EndPoint*in_RemoteEndPoint)
:ForwardedLink(in_Name, in_ParentLinkSet, in_LinkID, in_LocalEndPoint, in_RemoteEndPoint)
{
}

void QuakeLink::SendForwardedData(unsigned int in_SrcLANID, unsigned int in_SrcLinkID, unsigned int in_DstLANID, unsigned int in_DstLinkID, int in_ReplaceLocalPortAtOffset, const char*in_FwdBuf, int in_NumFwdBuf)
{
	if( in_ReplaceLocalPortAtOffset != -1 )
	{
		ForwardedLink*NewLink=dynamic_cast<ForwardedLink*>(GetParentProxy()->AddNewDynamicLinkTo(m_RemoteEndPoint));
		const EndPoint_UDP*NewLocalEndPoint=NewLink?dynamic_cast<const EndPoint_UDP*>(NewLink->GetLocalEndPoint()):NULL;
		if( NewLocalEndPoint != NULL )
		{
			//printf("%s: Accept message before (port=%d) = \n",GetName(), *((int*)(in_FwdBuf+in_ReplaceLocalPortAtOffset)));
			//HexDisplay( in_FwdBuf, in_NumFwdBuf );
			*((int*)(in_FwdBuf+in_ReplaceLocalPortAtOffset)) = (int)NewLocalEndPoint->GetPort();
			//printf("%s: Accept message after (port=%d) = \n",GetName(), *((int*)(in_FwdBuf+in_ReplaceLocalPortAtOffset)));
			//HexDisplay( in_FwdBuf, in_NumFwdBuf );
			NewLink->SetForwardingDestination(in_SrcLANID, in_SrcLinkID);
			NewLink->ForwardData(NULL, 0); // Send an empty forward message to get the routing table updated on the other side
		}
	}
	else
		SetForwardingDestination(in_SrcLANID, in_SrcLinkID);

	//DEBUG
	char*FwdBufCopy=new char[in_NumFwdBuf];
	memcpy(FwdBufCopy, in_FwdBuf, in_NumFwdBuf);
	int NumFwdBufCopy=in_NumFwdBuf;
	ProcessReceivedDataInternal(FwdBufCopy, NumFwdBufCopy, true);
	delete[]FwdBufCopy;
	//DEBUG END
	
	RemoteLink::SendForwardedData(in_SrcLANID, in_SrcLinkID, in_DstLANID, in_DstLinkID, in_ReplaceLocalPortAtOffset, in_FwdBuf, in_NumFwdBuf);
}

void QuakeLink::Work()
{
	ProcessReceivedData();
	RemoteLink::Work();
}
