/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "QuakeProxy.h"
#include "SingleLink.h"
#include "ListeningLinkSet.h"
#include "MultiLinkSets.h"

#include <strings.h>
#include <stdio.h>
#include <windows.h>

QuakeProxy::QuakeProxy(const char*in_Name)
:m_bValid(false), m_ControlLinkSet(NULL), m_QuakeLinkSet(NULL), m_QuakeServerEndPoint(NULL), m_UpdateTimeStamp(0)
{
	strncpy(m_Name, in_Name, 16);
	InitNetwork();
	char DLinkSetsName[QP_MAX_NAME];
	snprintf(DLinkSetsName, QP_MAX_NAME, "%s.%s", GetName(), "DLS");
	m_DynamicLinkSets = new MultiLinkSets(DLinkSetsName, this, 256);
}

void QuakeProxy::Client_Init(const EndPoint*in_LocalEndPoint,const EndPoint*in_RemoteEndPoint,const EndPoint*in_QuakeClientEndPoint)
{
	m_bValid=false;
	char CLinkSetName[QP_MAX_NAME], QLinkSetName[QP_MAX_NAME];
	snprintf(CLinkSetName, QP_MAX_NAME, "%s.%s", GetName(), "CCLS");
	snprintf(QLinkSetName, QP_MAX_NAME, "%s.%s", GetName(), "CQLS");
	m_ControlLinkSet = new SingleLink<TunnelLink>(CLinkSetName, this, in_LocalEndPoint, in_RemoteEndPoint);
	m_QuakeLinkSet   = new ListeningLinkSet<QuakeLink>(QLinkSetName, this, in_QuakeClientEndPoint, 256, QP_LAN_ID_SERVER, QP_LINK_ID_QUAKESERVER );
	AddRoutingEntry(QP_LAN_ID_SERVER, QP_LINK_ID_QUAKESERVER, m_ControlLinkSet->GetRemoteLinkByNum(m_ControlLinkSet->AddRemoteLinkTo(in_RemoteEndPoint))); 
	m_bValid = m_ControlLinkSet->IsValid() && m_QuakeLinkSet->IsValid();
	if(IsValid())
		printf("Created Client proxy.\n");
}

void QuakeProxy::Server_Init(const EndPoint*in_LocalEndPoint,const EndPoint*in_QuakeServerRemoteEndPoint)
{
	m_bValid=false;
	char CLinkSetName[QP_MAX_NAME], QLinkSetName[QP_MAX_NAME];
	snprintf(CLinkSetName, QP_MAX_NAME, "%s.%s", GetName(), "SCLS");
	snprintf(QLinkSetName, QP_MAX_NAME, "%s.%s", GetName(), "SQLS");
	m_ControlLinkSet = new LinkSet<TunnelLink>(CLinkSetName, this, in_LocalEndPoint, 256);
	//m_QuakeLinkSet   = new LinkSet<QuakeLink>(QLinkSetName, this, in_QuakeServerLocalEndPoint, 256);
	//m_QuakeLinkSet->AddRemoteLinkTo(in_QuakeServerRemoteEndPoint);
	m_QuakeServerEndPoint = in_QuakeServerRemoteEndPoint->Clone();
	m_bValid = m_ControlLinkSet->IsValid() && m_QuakeServerEndPoint->IsValid();
	if(IsValid())
		printf("Created Server proxy.\n");
}

void QuakeProxy::Server_AddClient(const EndPoint*in_RemoteEndPoint)
{
	m_ControlLinkSet->AddRemoteLinkTo(in_RemoteEndPoint);
}

void QuakeProxy::GetText(char*out_strText, const int in_BufLen) const
{
	if(!IsValid())
	{
		strncpy(out_strText, "Invalid Proxy", in_BufLen);
		return;
	}
	char ControlLinkSetText[2048], QuakeLinkSetText[2048], DynamicLinkSetsText[10240];
	if(m_ControlLinkSet != NULL)  m_ControlLinkSet->GetText(ControlLinkSetText, 2048);
	if(m_QuakeLinkSet != NULL)    m_QuakeLinkSet->GetText(QuakeLinkSetText, 2048);
	if(m_DynamicLinkSets != NULL) m_DynamicLinkSets->GetText(DynamicLinkSetsText, 10240);
	snprintf(out_strText, in_BufLen, "Proxy valid. LinkSet states:\n");
	snprintf(out_strText, in_BufLen, "%sControl LinkSet: %s\n", out_strText, m_ControlLinkSet?ControlLinkSetText:"NONE");
	snprintf(out_strText, in_BufLen, "%sQuake LinkSet: %s\n", out_strText, m_QuakeLinkSet?QuakeLinkSetText:"NONE");
	snprintf(out_strText, in_BufLen, "%sDynamic LinkSets: %s\n", out_strText, m_DynamicLinkSets?DynamicLinkSetsText:"NONE");
}

bool QuakeProxy::IsValid() const
{
	return m_bValid;
}

const char*QuakeProxy::GetName() const
{
	return m_Name;
}

int QuakeProxy::Run()
{
	while(true)
	{
		if(!IsValid())
			return 1;

		if(m_ControlLinkSet  != NULL) m_ControlLinkSet->Work();
		if(m_QuakeLinkSet    != NULL) m_QuakeLinkSet->Work();
		if(m_DynamicLinkSets != NULL) m_DynamicLinkSets->Work();

		if( false && GetTimeMSecs() - m_UpdateTimeStamp > 60000 ) 
		{
			char Desc[40960];
			GetText(Desc, 40960);
			printf("\nProxy State :\n%s\n", Desc);
			m_UpdateTimeStamp = GetTimeMSecs();
		}

		Sleep(1);
	};
	return 0;
}

RemoteLink*QuakeProxy::AddNewDynamicLinkTo( EndPoint*in_RemoteEndPoint )
{
	EndPoint_UDP ListenPoint;
	ListenPoint.SetLocalPort();
	return m_DynamicLinkSets->AddLink<LinkSet<QuakeLink> >(&ListenPoint, in_RemoteEndPoint);
}

RemoteLink*QuakeProxy::GetNextLinkInRoute(unsigned int in_LANID, unsigned int in_LinkID)
{
	if( m_QuakeServerEndPoint!=NULL && in_LANID==QP_LAN_ID_SELF && in_LinkID==QP_LINK_ID_QUAKESERVER )
		return AddNewDynamicLinkTo(m_QuakeServerEndPoint);

	RemoteLink* RoutingEntry = m_RoutingMap[ RoutingKey(in_LANID, in_LinkID) ];
	if( RoutingEntry != NULL )
		return RoutingEntry;

	printf("%s: Couldn't find a route to %d/%d.\n", GetName(), in_LANID, in_LinkID);

	return NULL;
}

void QuakeProxy::AddRoutingEntry(unsigned int in_LANID, unsigned int in_LinkID, RemoteLink*in_Link)
{
	m_RoutingMap[ RoutingKey( in_LANID, in_LinkID ) ] = in_Link;
}

QuakeProxy::~QuakeProxy()
{
	if(m_ControlLinkSet != NULL)
		delete m_ControlLinkSet;
	if(m_QuakeLinkSet != NULL)
		delete m_QuakeLinkSet;
}
