/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _QUAKEPROXY_H_
#define _QUAKEPROXY_H_

#include "LinkSet.h"
#include "TunnelLink.h"
#include "QuakeLink.h"
#include "ProxyBase.h"

#include <map>

class MultiLinkSets;

class QuakeProxy : public ProxyBase
{
	protected:
		bool     m_bValid;
		char     m_Name[16];

		LinkSet<TunnelLink>*m_ControlLinkSet;
		LinkSet<QuakeLink> *m_QuakeLinkSet;

		MultiLinkSets      *m_DynamicLinkSets;
		EndPoint           *m_QuakeServerEndPoint;

		struct RoutingKey
		{
			unsigned int LANID;
			unsigned int LinkID;
			RoutingKey(unsigned int in_LANID, unsigned int in_LinkID):LANID(in_LANID),LinkID(in_LinkID){}
			bool operator<(const RoutingKey in_Key)const{return(LANID<in_Key.LANID&&LinkID<in_Key.LinkID);}
		};
		std::map<RoutingKey, RemoteLink*> m_RoutingMap;

		int m_UpdateTimeStamp;
	public:
		QuakeProxy(const char*in_Name);
		RemoteLink*GetNextLinkInRoute(unsigned int in_LANID, unsigned int in_LinkID);
		virtual void AddRoutingEntry(unsigned int in_LANID, unsigned int in_LinkID, RemoteLink*in_Link);
		virtual RemoteLink*AddNewDynamicLinkTo( EndPoint*in_RemoteEndPoint );
		void Client_Init(const EndPoint*in_LocalEndPoint, const EndPoint*in_RemoteEndPoint, const EndPoint*in_QuakeClientEndPoint);
		void Server_Init(const EndPoint*in_LocalEndPoint, const EndPoint*in_QuakeServerRemoteEndPoint);
		void Server_AddClient(const EndPoint*in_RemoteEndPoint);
		void GetText(char*out_strText, const int in_BufLen) const;
		bool IsValid() const;
		const char*GetName() const;
		int Run();
		virtual ~QuakeProxy();
};

#endif // _QUAKEPROXY_H_
