/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _REMOTELINK_H_
#define _REMOTELINK_H_

#include "QP_Network.h"

class EndPoint;
class LinkSetBase;
struct sockaddr;
class ProxyBase;

class RemoteLink
{
	private:
		bool  m_bValid;
		char  m_Name[QP_MAX_NAME];

		int       m_NumSendBuffer;
		const int m_MaxSendBuffer;
		char     *m_SendBuffer;
	protected:
		int       m_NumRecvBuffer;
		const int m_MaxRecvBuffer;
		char     *m_RecvBuffer;

		const EndPoint*m_LocalEndPoint;
		EndPoint      *m_RemoteEndPoint;

		LinkSetBase   *m_ParentLinkSet;
		unsigned int   m_LinkID;
		virtual int GetPacketLength( const char*in_Data, int in_NumData ); 
	public:
		RemoteLink(const char*in_Name, LinkSetBase*in_ParentLinkSet, unsigned int in_LinkID, const EndPoint*in_LocalEndPoint, EndPoint*in_RemoteEndPoint);
		bool IsValid() const;
		const char*GetName() const;
		unsigned int GetLinkID() const;
		virtual void GetText(char*out_strText, const int in_BufLen) const;
		bool HasRemoteAddress(const EndPoint*in_RemoteEndPoint) const;
		void DoSendDataWith(int in_Socket);
		void ReceiveData(const char*in_RecvBuf, const int in_NumRecvBuf);
		void SendData(const char*in_SendBuf, const int in_NumSendBuf);
		virtual const char*GetTypeName() const;
		ProxyBase*GetParentProxy() const;
		virtual ~RemoteLink();
		virtual void Work();
		virtual void SendForwardedData(unsigned int in_SrcLANID, unsigned int in_SrcLinkID, unsigned int in_DstLANID, unsigned int in_DstLinkID, int in_ReplaceLocalPortAtOffset, const char*in_FwdBuf, int in_NumFwdBuf);
		const EndPoint*GetLocalEndPoint() const;
};

#endif // _REMOTELINK_H_
