/*
Copyright (C) 2006-2007 the DSQuake project.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "QuakeProxy.h"

#include <windows.h>
#include <stdio.h>

int GetTimeMSecs()
{
	#ifdef WIN32
		return GetTickCount();
	#else
		timeval t;
		gettimeofday(&t, 0);
		return t.tv_sec*1000+t.tv_usec/1000;
	#endif
}

int InitNetwork()
{
	#ifdef WIN32
		WSADATA wsaData;
		int err = WSAStartup(MAKEWORD( 2, 2 ), &wsaData);
		if ( err != 0 )
		{
		    MessageBox(0,"No Winsock for you", "WSAStartup failed", MB_OK);
		    return 1;
		}
	#endif
	return 0;
}

void Usage(char*progname)
{
	fprintf(stderr, "Usage : %s { cl ProxyServerIP[:ProxyServerPort] [QuakeClientListenPort] | sv QuakeServerIP[:QuakeServerPort] ProxyClientIP[:ProxyClientPort] }\n",progname);
}

int main (int argc, char *argv[])
{
	QuakeProxy Proxy("Proxy");
	if(argc<3)
	{
		Usage(argv[0]);
		return 1;
	}
	if(!strcmpi(argv[1], "cl")||!strcmpi(argv[1], "cld"))
	{
		EndPoint_UDP LocalEndPoint, RemoteEndPoint, QuakeClientEndPoint;
		RemoteEndPoint.SetAddress(argv[2], 20000 + (!strcmpi(argv[1], "cld")?100:0) );
		if(argc>=4)
			QuakeClientEndPoint.SetLocalPort(argv[3]);
		else
			QuakeClientEndPoint.SetLocalPort(26000);
		if(!RemoteEndPoint.IsValid() || !QuakeClientEndPoint.IsValid())
		{
			Usage(argv[0]);
			return 1;
		}
		LocalEndPoint.SetLocalPort(RemoteEndPoint.GetPort() + (!strcmpi(argv[1], "cld")?-100:0) );

		Proxy.Client_Init( &LocalEndPoint, &RemoteEndPoint, &QuakeClientEndPoint );
	}
	else if(!strcmpi(argv[1], "sv")||!strcmpi(argv[1], "svd"))
	{
		if(argc<4)
		{
			Usage(argv[0]);
			return 1;
		}
		EndPoint_UDP LocalEndPoint, RemoteEndPoint, QuakeServerRemoteEndPoint;
		RemoteEndPoint.SetAddress(argv[2], 20000);
		QuakeServerRemoteEndPoint.SetAddress(argv[3], 26000);
		if(!RemoteEndPoint.IsValid() || !QuakeServerRemoteEndPoint.IsValid())
		{
			Usage(argv[0]);
			return 1;
		}
		LocalEndPoint.SetLocalPort(RemoteEndPoint.GetPort()+ (!strcmpi(argv[1], "svd")?100:0) );

		Proxy.Server_Init( &LocalEndPoint, &QuakeServerRemoteEndPoint );
		Proxy.Server_AddClient( &RemoteEndPoint );
	}
	else
	{
		Usage(argv[0]);
		return 1;
	}

	return Proxy.Run();
}
